
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "descriptor/descriptor.h"


/* material */
ITEM mat_bender;	/* material of bender */
ITEM black,white;	/* color */
ITEM muro_attr;		/* */

/* bender */
ITEM corpo,braccioDx,braccioSx,gambe;
ITEM testa_l,
	testa,occhioDx,occhioSx,maschera,antenna,bocca,sigaro;

/* suicide booth */
ITEM suicideBooth,
	suicideBooth1,suicideBooth2,suicideBooth3,suicideBooth2x;

/* scene */
ITEM scene;

/* pavimento e pareti */
ITEM table,muro1,muro2;

float pos[3],targ[3],up[3],zoom;	/* camera setting */
char filemd[255];	/* file name for save md */
char filevw[255];	/* file name for save vw */

/* initialize camera */
void init_cam(){
	pos[0] = 12.0;
	pos[1] = 10.0;
	pos[2] = 2.0;

	targ[0] = 0.4;
	targ[1] = -0.7;
	targ[2] = 1.3;

	up[0] = 0;
	up[1] = 0;
	up[2] = 1;

	zoom = 32.0;
}

/* save current camera setting in the file filevw */
void save_cam(){
		FILE *fd;

		fd = fopen(filevw,"w");
		if(!fd){
			perror("Camera");
			exit(0);
		}
		fprintf(fd,"%f\t%f\t%f\n",pos[0],pos[1],pos[2]);
		fprintf(fd,"%f\t%f\t%f\n",targ[0],targ[1],targ[2]);
		fprintf(fd,"%f\t%f\t%f\n",up[0],up[1],up[2]);
		fprintf(fd,"%f\n",zoom);
		fprintf(fd,"1\n");

		fclose(fd);
}

/* set scene */
void set_ambient(){
  set_background(0.676,0.861,0.953,FALSE);
  set_ambient_light(1,1,1,0.7);
  create_point_light(11.0,11.0,11.0,1,1,1,4,20, "light1");
  create_point_light(11.0,-11.0,11.0,1,1,1,4,20, "light2");
  create_point_light(15.0,0.0,15.0,1,1,1,4,25, "light3");
}

/* init attribute */
void init_attr(){
	mat_bender =  create_attribute("bender");
	//set_color(mat_bender,0,0,1,0.5,0.5);
	//set_reflectivity(mat_bender,0.3,10);

	black = create_attribute("black");
	set_color(black,0,0,0,0.2,0.5);
	set_reflectivity(black, 0,0);

	white = create_attribute("white");
	set_color(white,1,1,1,0.2,0.5);
	set_reflectivity(white, 0,0);

	muro_attr = create_attribute("muro");
	set_color(muro_attr,0.8,0.8,0.8,0.590,0.661);
	set_reflectivity(muro_attr,0, 0);
	set_transparency(muro_attr,0,0,0,6.519);
}

int main(void) {

	init_cam();	/* initialize camera */
	strcpy(filevw,"bender.vw");	/* initialize name of the file to store camera setting */
	save_cam();			/* save camera setting */
	set_ambient();			/* set the ambient light and background */
	init_attr();			/* inizialize attribute */

	/* BUILD SCENE */

	//testa = read_obj("testa.obj", "testa");
	testa = read_nurbs("testaS.db");
	set_attribute(testa,mat_bender);
	testa_l = create_list(testa,"testaL");

	occhioDx = read_nurbs("occhioDxS.db");
	set_attribute(testa,mat_bender);
	testa_l = insert_in_list(testa_l,occhioDx);

	occhioSx = read_nurbs("occhioSxS.db");
	set_attribute(occhioSx,mat_bender);
	testa_l = insert_in_list(testa_l,occhioSx);

	maschera = read_nurbs("mascheraS.db");
	set_attribute(occhioSx,mat_bender);
	testa_l = insert_in_list(testa_l,maschera);

	antenna = read_nurbs("antennaS.db");
	set_attribute(occhioSx,mat_bender);
	testa_l = insert_in_list(testa_l,antenna);
	
	bocca = read_nurbs("boccaS.db");
	testa_l = insert_in_list(testa_l,bocca);

	sigaro = read_nurbs("sigaroS.db");
	testa_l = insert_in_list(testa_l,sigaro);

	corpo = read_obj("corpo.obj","corpo");
	braccioDx = read_obj("braccioDx.obj","braccioDx");
	braccioSx = read_obj("braccioSx.obj","braccioSx");

	gambe = read_obj("gambe.obj","gambe");
	set_attribute(gambe,mat_bender);

	//suicideBooth = read_obj("suicideBooth.obj","suicideBooth");
	suicideBooth1 = read_nurbs("suicideBooth1S.db");
	set_attribute(suicideBooth1,mat_bender);
	set_scale(suicideBooth1,4,4,4);
	set_translate(suicideBooth1,-4,-4,-2);

	suicideBooth2 = read_nurbs("suicideBooth2S.db");
	set_attribute(suicideBooth2,mat_bender);
	set_scale(suicideBooth2,4,4,4);
	set_translate(suicideBooth2,-4,-4,-2);

	suicideBooth3 = read_nurbs("suicideBooth3S.db");
	set_attribute(suicideBooth3,mat_bender);
	set_scale(suicideBooth3,4,4,4);
	set_translate(suicideBooth3,-4,-4,-2);

	suicideBooth2x = read_obj("suicideBooth2.obj","suicideBooth2x");
	set_attribute(suicideBooth2x,mat_bender);
	set_scale(suicideBooth2x,4,4,4);
	set_translate(suicideBooth2x,-4,-4,-2);

	suicideBooth = create_list(suicideBooth1,"suicideBooth");
	suicideBooth = insert_in_list(suicideBooth,suicideBooth2);
	suicideBooth = insert_in_list(suicideBooth,suicideBooth3);
	suicideBooth = insert_in_list(suicideBooth,suicideBooth2x);
	

	table = read_nurbs("table.db");
	muro1 = create_copy(table,"muro1");
	set_attribute(muro1,muro_attr);
	muro2 = create_copy(table,"muro2");
	set_attribute(muro2,muro_attr);

	set_z_rotate(table,45,1);
	set_translate(table,20,20,0);
	set_cube_texture(table,black,white);
	set_translate(table,-20,-20,-2);
	set_z_rotate(table,-45,1);

	set_y_rotate(muro1,90,1);
	set_scale(muro1,1,1,0.5);
	set_translate(muro1,-10,0,3);

	set_x_rotate(muro2,90,1);
	set_scale(muro2,1,1,0.5);
	set_translate(muro2,0,-10,3);


	/*	TEXTURING	*/

	set_domain_texture(bocca,mat_bender,"textures/tex_bocca.hr",0,1,0,1,FALSE,1,1);
	set_domain_texture(sigaro,mat_bender,"textures/tex_sigaro.hr",0,1,0,1,FALSE,0.5,1);
	set_domain_texture(occhioDx,mat_bender,"textures/tex_occhi.hr",0,1,0,1,FALSE,0.5,1);
	set_domain_texture(occhioSx,mat_bender,"textures/tex_occhi.hr",0,1,0,1,FALSE,0.5,1);
	set_domain_texture(suicideBooth1,mat_bender,"textures/tex_suicide.hr",0,1,0,1,FALSE,0.5,1);
	set_domain_texture(suicideBooth2,mat_bender,"textures/tex_suicide1.hr",0,1,0,1,FALSE,0.5,1);
	set_domain_texture(suicideBooth3,mat_bender,"textures/tex_suicide2.hr",0,1,0,1,FALSE,0.5,1);

	set_bump_texture(testa,mat_bender,"textures/bump_bocca.hr",0.0f,1.0f,0.0f,1.0f,FALSE, 1,1);
	set_bump_texture(braccioDx,mat_bender,"textures/bump_righe.hr",0.0f,1.0f,0.0f,1.0f,FALSE,1,-1);
	set_bump_texture(braccioSx,mat_bender,"textures/bump_righe.hr",0.0f,1.0f,0.0f,1.0f,FALSE,1,-1);
	set_bump_texture(gambe,mat_bender,"textures/bump_righe2.hr",0.0f,1.0f,0.0f,1.0f,FALSE,1,-1);
	set_bump_texture(corpo,mat_bender,"textures/bump_corpo.hr",0.0f,1.0f,0.0f,1.0f,FALSE,1,-1);

	/*	CREATE SCENE LIST	*/
	scene = create_list(table, "scene");
	scene = insert_in_list(scene,testa_l);
	scene = insert_in_list(scene,corpo);
	scene = insert_in_list(scene,braccioDx);
	scene = insert_in_list(scene,braccioSx);
	scene = insert_in_list(scene,gambe);
	scene = insert_in_list(scene,suicideBooth);
	scene = insert_in_list(scene,muro1);
	scene = insert_in_list(scene,muro2);

		/* braccia giu */

		set_y_rotate(braccioSx,180,1);
		set_y_rotate(braccioDx,180,1);
		set_translate(braccioDx,0.19,0,-0.67);
		set_translate(braccioSx,0.03,0,-0.66);

	save_scene("bender.md","scene_bender",scene);

  return(0);
}
